#!/usr/bin/env python3
"""
#exonware/xwquery/tests/strategies/__init__.py
Test strategies package for XWQuery universal query language testing.
Company: eXonware.com
Author: eXonware Backend Team
Email: connect@exonware.com
Version: 0.0.1
Generation Date: 28-Oct-2025
"""
# Test infrastructure for 31 query strategies

__all__ = [
    'BaseStrategyTest',
    'generate_strategy_tests'
]
